@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2009,2013 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2015,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -30,10 +30,11 @@
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
  ****************************************************************************/
 
-/* $Id: form.h,v 0.23 2013/12/07 17:57:32 tom Exp $ */
+/* $Id: form.h,v 0.26 2016/10/29 22:24:24 Leon.Winter Exp $ */
 
 #ifndef FORM_H
 #define FORM_H
+/* *INDENT-OFF*/
 
 #include <curses.h>
 #include <eti.h>
@@ -203,6 +204,8 @@ typedef void (*Form_Hook)(FORM *);
 #define O_NULLOK		(0x0080U)
 #define O_PASSOK		(0x0100U)
 #define O_STATIC		(0x0200U)
+#define O_DYNAMIC_JUSTIFY	(0x0400U)	/* ncurses extension	*/
+#define O_NO_LEFT_STRIP		(0x0800U)	/* ncurses extension	*/
 
 /* form options */
 #define O_NL_OVERLOAD		(0x0001U)
@@ -384,6 +387,7 @@ extern NCURSES_EXPORT(int)	field_count (const FORM *);
 extern NCURSES_EXPORT(int)	set_form_win (FORM *,WINDOW *);
 extern NCURSES_EXPORT(int)	set_form_sub (FORM *,WINDOW *);
 extern NCURSES_EXPORT(int)	set_current_field (FORM *,FIELD *);
+extern NCURSES_EXPORT(int)	unfocus_current_field (FORM *);
 extern NCURSES_EXPORT(int)	field_index (const FIELD *);
 extern NCURSES_EXPORT(int)	set_form_page (FORM *,int);
 extern NCURSES_EXPORT(int)	form_page (const FORM *);
@@ -421,5 +425,6 @@ extern NCURSES_EXPORT(FORM *)	NCURSES_SP_NAME(new_form) (SCREEN*, FIELD **);
 #ifdef __cplusplus
   }
 #endif
+/* *INDENT-ON*/
 
-#endif	/* FORM_H */
+#endif /* FORM_H */
