@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998,2000 Free Software Foundation, Inc.                   *
+ * Copyright (c) 1998-2009,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -28,9 +28,10 @@
 
 /****************************************************************************
  *   Author:  Juergen Pfeifer, 1995,1997                                    *
- *   Contact: http://www.familiepfeifer.de/Contact.aspx?Lang=en             *
  ****************************************************************************/
 
+/* $Id: menu.h,v 1.21 2016/03/26 21:52:08 tom Exp $ */
+
 #ifndef ETI_MENU
 #define ETI_MENU
 
@@ -55,6 +56,7 @@ typedef int Item_Options;
 #define O_IGNORECASE    (0x08)
 #define O_SHOWMATCH     (0x10)
 #define O_NONCYCLIC     (0x20)
+#define O_MOUSE_MENU    (0x40)
 
 /* Item options: */
 #define O_SELECTABLE    (0x01)
@@ -77,7 +79,7 @@ typedef struct tagITEM
   short          x;
   bool           value;       /* Selection value                           */
                              
-  struct tagITEM *left;       /* neighbour items                           */
+  struct tagITEM *left;       /* neighbor items                            */
   struct tagITEM *right;
   struct tagITEM *up;
   struct tagITEM *down;
@@ -216,23 +218,23 @@ extern NCURSES_EXPORT(int)	pos_menu_cursor (const MENU *);
 extern NCURSES_EXPORT(int)	post_menu (MENU *);
 extern NCURSES_EXPORT(int)	scale_menu (const MENU *,int *,int *);
 extern NCURSES_EXPORT(int)	set_current_item (MENU *menu,ITEM *item);
-extern NCURSES_EXPORT(int)	set_item_init (MENU *,void(*)(MENU *));
+extern NCURSES_EXPORT(int)	set_item_init (MENU *, Menu_Hook);
 extern NCURSES_EXPORT(int)	set_item_opts (ITEM *,Item_Options);
-extern NCURSES_EXPORT(int)	set_item_term (MENU *,void(*)(MENU *));
+extern NCURSES_EXPORT(int)	set_item_term (MENU *, Menu_Hook);
 extern NCURSES_EXPORT(int)	set_item_userptr (ITEM *, void *);
 extern NCURSES_EXPORT(int)	set_item_value (ITEM *,bool);
 extern NCURSES_EXPORT(int)	set_menu_back (MENU *,chtype);
 extern NCURSES_EXPORT(int)	set_menu_fore (MENU *,chtype);
 extern NCURSES_EXPORT(int)	set_menu_format (MENU *,int,int);
 extern NCURSES_EXPORT(int)	set_menu_grey (MENU *,chtype);
-extern NCURSES_EXPORT(int)	set_menu_init (MENU *,void(*)(MENU *));
+extern NCURSES_EXPORT(int)	set_menu_init (MENU *, Menu_Hook);
 extern NCURSES_EXPORT(int)	set_menu_items (MENU *,ITEM **);
 extern NCURSES_EXPORT(int)	set_menu_mark (MENU *, const char *);
 extern NCURSES_EXPORT(int)	set_menu_opts (MENU *,Menu_Options);
 extern NCURSES_EXPORT(int)	set_menu_pad (MENU *,int);
 extern NCURSES_EXPORT(int)	set_menu_pattern (MENU *,const char *);
 extern NCURSES_EXPORT(int)	set_menu_sub (MENU *,WINDOW *);
-extern NCURSES_EXPORT(int)	set_menu_term (MENU *,void(*)(MENU *));
+extern NCURSES_EXPORT(int)	set_menu_term (MENU *, Menu_Hook);
 extern NCURSES_EXPORT(int)	set_menu_userptr (MENU *,void *);
 extern NCURSES_EXPORT(int)	set_menu_win (MENU *,WINDOW *);
 extern NCURSES_EXPORT(int)	set_top_row (MENU *,int);
@@ -248,6 +250,10 @@ extern NCURSES_EXPORT(bool)	item_visible (const ITEM *);
 
 extern NCURSES_EXPORT(void)	menu_format (const MENU *,int *,int *);
 
+#if NCURSES_SP_FUNCS
+extern NCURSES_EXPORT(MENU *)	NCURSES_SP_NAME(new_menu) (SCREEN*, ITEM **);
+#endif
+
 #ifdef __cplusplus
   }
 #endif
