@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2010,2012 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2014,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -39,14 +39,12 @@
 #include <curses.priv.h>
 #include <termcap.h>		/* ospeed */
 
-MODULE_ID("$Id: lib_cur_term.c,v 1.31 2012/11/18 02:12:43 tom Exp $")
+MODULE_ID("$Id: lib_cur_term.c,v 1.34 2016/05/28 23:22:52 tom Exp $")
 
 #undef CUR
 #define CUR termp->type.
 
-#if BROKEN_LINKER && !USE_REENTRANT
-NCURSES_EXPORT_VAR(TERMINAL *) cur_term = 0;
-#elif BROKEN_LINKER || USE_REENTRANT
+#if USE_REENTRANT
 
 NCURSES_EXPORT(TERMINAL *)
 NCURSES_SP_NAME(_nc_get_cur_term) (NCURSES_SP_DCL0)
@@ -55,6 +53,7 @@ NCURSES_SP_NAME(_nc_get_cur_term) (NCURSES_SP_DCL0)
 }
 
 #if NCURSES_SP_FUNCS
+
 NCURSES_EXPORT(TERMINAL *)
 _nc_get_cur_term(void)
 {
@@ -87,10 +86,10 @@ NCURSES_SP_NAME(set_curterm) (NCURSES_SP_DCLx TERMINAL * termp)
     oldterm = cur_term;
     if (SP_PARM)
 	SP_PARM->_term = termp;
-#if BROKEN_LINKER && !USE_REENTRANT
-    cur_term = termp;
-#else
+#if USE_REENTRANT
     CurTerm = termp;
+#else
+    cur_term = termp;
 #endif
     if (termp != 0) {
 #ifdef USE_TERM_DRIVER
@@ -133,9 +132,7 @@ NCURSES_SP_NAME(del_curterm) (NCURSES_SP_DCLx TERMINAL * termp)
 	TERMINAL_CONTROL_BLOCK *TCB = (TERMINAL_CONTROL_BLOCK *) termp;
 #endif
 	TERMINAL *cur = (
-#if BROKEN_LINKER && !USE_REENTRANT
-			    cur_term
-#elif BROKEN_LINKER || USE_REENTRANT
+#if USE_REENTRANT
 			    NCURSES_SP_NAME(_nc_get_cur_term) (NCURSES_SP_ARG)
 #else
 			    cur_term
@@ -154,7 +151,7 @@ NCURSES_SP_NAME(del_curterm) (NCURSES_SP_DCLx TERMINAL * termp)
 #endif
 #ifdef USE_TERM_DRIVER
 	if (TCB->drv)
-	    TCB->drv->release(TCB);
+	    TCB->drv->td_release(TCB);
 #endif
 	free(termp);
 
@@ -167,7 +164,7 @@ NCURSES_SP_NAME(del_curterm) (NCURSES_SP_DCLx TERMINAL * termp)
 NCURSES_EXPORT(int)
 del_curterm(TERMINAL * termp)
 {
-    int rc = ERR;
+    int rc;
 
     _nc_lock_global(curses);
     rc = NCURSES_SP_NAME(del_curterm) (CURRENT_SCREEN, termp);
