@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2003,2009 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2009,2013 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -46,16 +46,16 @@
 #define CUR SP_TERMTYPE
 #endif
 
-MODULE_ID("$Id: lib_has_cap.c,v 1.8 2009/05/10 00:53:52 tom Exp $")
+MODULE_ID("$Id: lib_has_cap.c,v 1.10 2013/11/16 19:57:22 tom Exp $")
 
 NCURSES_EXPORT(bool)
 NCURSES_SP_NAME(has_ic) (NCURSES_SP_DCL0)
 {
     bool code = FALSE;
 
-    T((T_CALLED("has_ic(%p)"), SP_PARM));
+    T((T_CALLED("has_ic(%p)"), (void *) SP_PARM));
 
-    if (IsValidTIScreen(SP_PARM) && IsTermInfo(SP_PARM)) {
+    if (HasTInfoTerminal(SP_PARM)) {
 	code = ((insert_character || parm_ich
 		 || (enter_insert_mode && exit_insert_mode))
 		&& (delete_character || parm_dch)) ? TRUE : FALSE;
@@ -76,8 +76,8 @@ NCURSES_EXPORT(bool)
 NCURSES_SP_NAME(has_il) (NCURSES_SP_DCL0)
 {
     bool code = FALSE;
-    T((T_CALLED("has_il(%p)"), SP_PARM));
-    if (IsValidTIScreen(SP_PARM) && IsTermInfo(SP_PARM)) {
+    T((T_CALLED("has_il(%p)"), (void *) SP_PARM));
+    if (HasTInfoTerminal(SP_PARM)) {
 	code = ((insert_line || parm_insert_line)
 		&& (delete_line || parm_delete_line)) ? TRUE : FALSE;
     }
