@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2006,2007 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2013,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -36,16 +36,10 @@
  * clear.c --  clears the terminal's screen
  */
 
-#define USE_LIBTINFO
-#include <progs.priv.h>
+#define USE_LIBTINFO 1
+#include <clear_cmd.h>
 
-MODULE_ID("$Id: clear.c,v 1.11 2007/10/13 22:16:02 tom Exp $")
-
-static int
-putch(int c)
-{
-    return putchar(c);
-}
+MODULE_ID("$Id: clear.c,v 1.15 2016/10/23 00:36:36 tom Exp $")
 
 int
 main(
@@ -53,7 +47,8 @@ main(
 	char *argv[]GCC_UNUSED)
 {
     setupterm((char *) 0, STDOUT_FILENO, (int *) 0);
-    ExitProgram((tputs(clear_screen, lines > 0 ? lines : 1, putch) == ERR)
+
+    ExitProgram((clear_cmd() == ERR)
 		? EXIT_FAILURE
 		: EXIT_SUCCESS);
 }
