@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 2005-2013,2014 Free Software Foundation, Inc.              *
+ * Copyright (c) 2005-2015,2016 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -29,7 +29,7 @@
 /*
  * Author: Thomas E. Dickey
  *
- * $Id: demo_termcap.c,v 1.45 2014/09/05 08:44:49 tom Exp $
+ * $Id: demo_termcap.c,v 1.51 2016/09/10 21:22:39 tom Exp $
  *
  * A simple demo of the termcap interface.
  */
@@ -46,6 +46,12 @@
 #endif
 #endif
 
+#ifdef NCURSES_VERSION
+#include <termcap.h>
+#endif
+
+static void failed(const char *) GCC_NORETURN;
+
 static void
 failed(const char *msg)
 {
@@ -94,8 +100,9 @@ static long total_s_values;
 static char *
 make_dbitem(char *p, char *q)
 {
-    char *result = malloc(strlen(e_opt) + 2 + (size_t) (p - q));
-    sprintf(result, "%s=%.*s", e_opt, (int) (p - q), q);
+    size_t need = strlen(e_opt) + 2 + (size_t) (p - q);
+    char *result = malloc(need);
+    _nc_SPRINTF(result, _nc_SLIMIT(need) "%s=%.*s", e_opt, (int) (p - q), q);
     return result;
 }
 
@@ -162,6 +169,72 @@ free_dblist(void)
     }
 }
 
+static void
+show_string(const char *name, const char *value)
+{
+    printf(FNAME(str), name);
+    if (value == ((char *) -1)) {
+	printf("CANCELLED");
+    } else if (value == ((char *) 0)) {
+	printf("ABSENT");
+    } else {
+	while (*value != 0) {
+	    int ch = UChar(*value++);
+	    switch (ch) {
+	    case '\177':
+		fputs("^?", stdout);
+		break;
+	    case '\033':
+		fputs("\\E", stdout);
+		break;
+	    case '\b':
+		fputs("\\b", stdout);
+		break;
+	    case '\f':
+		fputs("\\f", stdout);
+		break;
+	    case '\n':
+		fputs("\\n", stdout);
+		break;
+	    case '\r':
+		fputs("\\r", stdout);
+		break;
+	    case ' ':
+		fputs("\\s", stdout);
+		break;
+	    case '\t':
+		fputs("\\t", stdout);
+		break;
+	    case '^':
+		fputs("\\^", stdout);
+		break;
+	    case ':':
+		fputs("\\072", stdout);
+		break;
+	    case '\\':
+		fputs("\\\\", stdout);
+		break;
+	    default:
+		if (isgraph(ch))
+		    fputc(ch, stdout);
+		else if (ch < 32)
+		    printf("^%c", ch + '@');
+		else
+		    printf("\\%03o", ch);
+		break;
+	    }
+	}
+    }
+    printf("\n");
+}
+
+static void
+show_number(const char *name, int value)
+{
+    printf(FNAME(num), name);
+    printf(" %d\n", value);
+}
+
 static void
 dumpit(NCURSES_CONST char *cap)
 {
@@ -184,61 +257,13 @@ dumpit(NCURSES_CONST char *cap)
 	     * Note that the strings returned are mostly terminfo format, since
 	     * ncurses does not convert except for a handful of special cases.
 	     */
-	    printf(FNAME(str), cap);
-	    while (*str != 0) {
-		int ch = UChar(*str++);
-		switch (ch) {
-		case '\177':
-		    fputs("^?", stdout);
-		    break;
-		case '\033':
-		    fputs("\\E", stdout);
-		    break;
-		case '\b':
-		    fputs("\\b", stdout);
-		    break;
-		case '\f':
-		    fputs("\\f", stdout);
-		    break;
-		case '\n':
-		    fputs("\\n", stdout);
-		    break;
-		case '\r':
-		    fputs("\\r", stdout);
-		    break;
-		case ' ':
-		    fputs("\\s", stdout);
-		    break;
-		case '\t':
-		    fputs("\\t", stdout);
-		    break;
-		case '^':
-		    fputs("\\^", stdout);
-		    break;
-		case ':':
-		    fputs("\\072", stdout);
-		    break;
-		case '\\':
-		    fputs("\\\\", stdout);
-		    break;
-		default:
-		    if (isgraph(ch))
-			fputc(ch, stdout);
-		    else if (ch < 32)
-			printf("^%c", ch + '@');
-		    else
-			printf("\\%03o", ch);
-		    break;
-		}
-	    }
-	    printf("\n");
+	    show_string(cap, str);
 	}
     } else if ((num = tgetnum(cap)) >= 0) {
 	total_values++;
 	total_n_values++;
 	if (!q_opt) {
-	    printf(FNAME(num), cap);
-	    printf(" %d\n", num);
+	    show_number(cap, num);
 	}
     } else if (tgetflag(cap) > 0) {
 	total_values++;
@@ -405,6 +430,7 @@ parse_description(const char *input_name)
     if ((fp = fopen(input_name, "r")) == 0)
 	failed("cannot open input-file");
     len = fread(my_blob, sizeof(char), (size_t) sb.st_size, fp);
+    my_blob[sb.st_size] = '\0';
     fclose(fp);
 
     /*
@@ -674,7 +700,7 @@ copy_code_list(NCURSES_CONST char *const *list)
 		length += chunk;
 	    } else {
 		result[count] = unused;
-		strcpy(unused, list[count]);
+		_nc_STRCPY(unused, list[count], length);
 		unused += chunk;
 	    }
 	}
@@ -711,6 +737,7 @@ usage(void)
 	" -q       quiet (prints only counts)",
 	" -r COUNT repeat for given count",
 	" -s       print string-capabilities",
+	" -v       print termcap-variables",
 #ifdef NCURSES_VERSION
 	" -x       print extended capabilities",
 #endif
@@ -728,12 +755,13 @@ main(int argc, char *argv[])
     int n;
     char *name;
     bool a_opt = FALSE;
+    bool v_opt = FALSE;
     char *input_name = 0;
 
     int repeat;
     int r_opt = 1;
 
-    while ((n = getopt(argc, argv, "abd:e:i:nqr:sxy")) != -1) {
+    while ((n = getopt(argc, argv, "abd:e:i:nqr:svxy")) != -1) {
 	switch (n) {
 	case 'a':
 	    a_opt = TRUE;
@@ -763,6 +791,9 @@ main(int argc, char *argv[])
 	case 's':
 	    s_opt = TRUE;
 	    break;
+	case 'v':
+	    v_opt = TRUE;
+	    break;
 #if NCURSES_XNAMES
 	case 'x':
 	    x_opt = TRUE;
@@ -835,6 +866,15 @@ main(int argc, char *argv[])
     printf("%ld values (%ld booleans, %ld numbers, %ld strings)\n",
 	   total_values, total_b_values, total_n_values, total_s_values);
 
+#if defined(NCURSES_VERSION) || defined(HAVE_CURSES_DATA_OSPEED)
+    if (v_opt) {
+	show_number("PC", PC);
+	show_string("UP", UP);
+	show_string("BC", BC);
+	show_number("ospeed", ospeed);
+    }
+#endif
+
     free_dblist();
 
     ExitProgram(EXIT_SUCCESS);
