@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright (c) 1998-2007,2008 Free Software Foundation, Inc.              *
+ * Copyright (c) 1998-2010,2013 Free Software Foundation, Inc.              *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
  * copy of this software and associated documentation files (the            *
@@ -30,7 +30,7 @@
  *
  * Generate timing statistics for vertical-motion optimization.
  *
- * $Id: hashtest.c,v 1.29 2008/08/16 17:26:44 tom Exp $
+ * $Id: hashtest.c,v 1.32 2013/04/27 19:50:17 tom Exp $
  */
 
 #include <test.priv.h>
@@ -54,7 +54,7 @@ cleanup(void)
     endwin();
 }
 
-static RETSIGTYPE
+static void
 finish(int sig GCC_UNUSED)
 {
     cleanup();
@@ -86,7 +86,8 @@ genlines(int base)
 
     move(head_lines, 0);
     for (i = head_lines; i < LINES - foot_lines; i++) {
-	chtype c = (base - LO_CHAR + i) % (HI_CHAR - LO_CHAR + 1) + LO_CHAR;
+	chtype c = (chtype) ((base - LO_CHAR + i) % (HI_CHAR - LO_CHAR + 1)
+			     + LO_CHAR);
 	int hi = (extend_corner || (i < LINES - 1)) ? COLS : COLS - 1;
 	for (j = 0; j < hi; j++)
 	    addch(c);
@@ -140,10 +141,10 @@ run_test(bool optimized GCC_UNUSED)
 #endif
 
     if (reverse_loops)
-	for (ch = hi; ch >= lo; ch--)
+	for (ch = (char) hi; ch >= lo; ch--)
 	    one_cycle(ch);
     else
-	for (ch = lo; ch <= hi; ch++)
+	for (ch = (char) lo; ch <= hi; ch++)
 	    one_cycle(ch);
 }
 
